--[[
************************************************************************
Project				: BetterGuildControl
Author				: zhinjio
Project Revision	: r14
Project Date		: 20090814201122

File				: BetterGuildControl.lua
Commit Author		: zhinjio
Commit Revision		: 14
Commit Date			: 20090814201122
************************************************************************
Description	:
	Main addon file. 

TODO		:

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local MODNAME = "BetterGuildControl"
local FULLNAME = "BetterGuildControl"

local BetterGuildControl = LibStub( "AceAddon-3.0" ):GetAddon( MODNAME )
local L = LibStub:GetLibrary( "AceLocale-3.0" ):GetLocale( MODNAME )

local AceConfig 		= LibStub("AceConfig-3.0")
local AceConfigReg 		= LibStub("AceConfigRegistry-3.0")
local AceConfigDialog 	= LibStub("AceConfigDialog-3.0")

local addon = BetterGuildControl

local addonversion = "r14"
local addonauthor = "zhinjio"
local builddate = "20090814201122"

local modularOptions = {}
local tooltip
local LTT = LibStub:GetLibrary( "LibQTip-1.0" )

local db

--[[
local checkicon = "\124TInterface\\RAIDFRAME\\ReadyCheck-Ready:14\124t"
local xicon = "\124TInterface\\RAIDFRAME\\ReadyCheck-NotReady:14\124t"
]]--

-- coloring agents
--[[
local function Orange ( text ) return "|cffdea51c" .. text .. "|r" end
local function Yellow ( text ) return "|cffffff00" .. text .. "|r" end
local function Cyan ( text ) return "|cff00ffff" .. text .. "|r" end
]]--

local function drawTooltip()
	tooltip:Hide()
	tooltip:Clear()
	local numCols = 1
	local lineNum = tooltip:AddLine( "b" )
	if ( addon.dataReady == false ) then
		tooltip:SetCell( lineNum, 1, L["You are not currently in a guild"], "LEFT", numCols )
		lineNum = tooltip:AddLine( "c" )
		tooltip:SetCell( lineNum, 1, L["Alternately, guild information may not yet be available"], "LEFT", numCols )
	else
		tooltip:SetCell( lineNum, 1, L["Left-Click to open main window"], "LEFT", numCols)
		lineNum = tooltip:AddLine( "c" )
		tooltip:SetCell( lineNum, 1, L["Right-click for Configuration"], "LEFT", numCols )
	end
end

-- LDB stuff
local LDB = LibStub( "LibDataBroker-1.1" )
local launcher = LDB:NewDataObject( MODNAME, {
	type = "launcher",
	text = " ",
	label = FULLNAME,
	icon = "Interface\\Addons\\" .. FULLNAME .. "\\icon",
	OnClick = function( clickedframe, button )
				if ( addon.dataReady == true ) then
					if button == "RightButton" then
						addon:ShowConfig()
					else
						addon:ToggleMainPanel()
					end
				end
	end,
	OnEnter = function ( self )
		tooltip = LTT:Acquire( MODNAME.."TT", 1, "LEFT" )
		tooltip:SetAutoHideDelay( .1, self )
		if ( addon.dataReady == true ) then
			tooltip:SetScale( db.options.display.tipscale )
		end
		drawTooltip()
		tooltip:EnableMouse()
		tooltip:SmartAnchorTo( self )
		tooltip:Show()
	end,
	OnLeave = function ( self )
		-- stub. autohidedelay should handle frame recycling
		return
	end,
} )

function addon.upgradeDB()
	local olddb = addon.db.factionrealm[addon.guild]
	if ( olddb.options.display.dbVersion == "1.0" ) then
		olddb.options.display.dbVersion = "1.1"
		olddb.players = {}
		olddb.toons = {}
		-- Need to get rid of the old addon.me stuff
		baddb = addon.db.factionrealm[UnitName("player")]
		baddb = nil
		addon.Print( string.format( L["Database upgraded to %s"], "1.1" ) )
	end
end

-- options stuff
local displayOptions = nil
local function giveDisplay()
	if (not displayOptions) then
		displayOptions = {
			order = 2,
			type = "group",
			name = L["Display Options"],
			desc = L["DISPLAY_OPTIONS_DESC"],
			args = {
				header1 = {
					order	= 21,
					type	= "header",
					name	= L["Display Options"],
				},
				desc = {
					order	= 22,
					type	= "description",
					name	= L["DISPLAY_OPTIONS_DESC"],
				},
				sound = {
					order	= 23,
					type	= "toggle",
					name	= L["Enable DEBUG"],
					desc	= L["DISPLAY_DEBUG_DESC"],
					get		= function() return db.options.display.debug end,
					set		= function() db.options.display.debug = not db.options.display.debug end,
				},
				tooltipscale = {
					order	= 24,
					type	= "range",
					name	= L["Tooltip Scale"],
					desc	= L["TOOLTIP_SCALE_DESC"],
					min		= .7,
					max		= 1.3,
					step	= .05,
					bigStep = .05,
					get		= function() return db.options.display.tipscale end,
					set		= function( info, v ) 
								db.options.display.tipscale = v
								tooltip:SetScale( v )
							  end,
				},
			},
		}
	end
	return displayOptions
end

local options = nil
local function fullOptions()
	if (not options) then
		options = {
			type = "group",
			name = MODNAME,
			args = {
				general = {
					order	= 1,
					type	= "group",
					name	= L["General Information"],
					desc	= L["GENERAL_INFO_DESC"],
					args	= {
						header1 = {
							order	= 11,
							type	= "header",
							name	= L["General Information"],
						},
						desc = {
							order	= 12,
							type	= "description",
							name	= L["GENERAL_INFO_DESC"],
						},
						author = {
							order	= 13,
							type	= "description",
							name	= L["Author : "] .. addonauthor .. "\n",
						},
						version = {
							order	= 14,
							type	= "description",
							name	= L["Version : "] .. addonversion .. "\n",
						},
						builddate = {
							order	= 15,
							type	= "description",
							name	= L["Build Date : "] .. builddate .. "\n",
						},
						dbversion = {
							order	= 15,
							type	= "description",
							name	= L["Database Version : "] .. db.options.display.dbVersion .. "\n",
						},
						translators = {
							order	= 16,
							type	= "description",
							name	= string.format( L["Helpful Translators (thank you) : %s"], " \n" )
						},
					},
				},
			},
		}
	end
	
	for k,v in pairs(modularOptions) do
		options.args[k] = (type(v) == "function") and v() or v
	end

	return options
end

function addon:SetupOptions()
	AceConfigReg:RegisterOptionsTable(MODNAME, fullOptions)
	self.optionsFrame = AceConfigDialog:AddToBlizOptions(MODNAME, nil, nil, "general")

	-- Fill up our modular options...
	self:RegisterModuleOptions("Display", giveDisplay(), L["Display Options"])
end

function addon:RegisterModuleOptions(name, optionsTable, displayName)
	modularOptions[name] = optionsTable
	self.optionsFrame[name] = AceConfigDialog:AddToBlizOptions(MODNAME, displayName, MODNAME, name)
end

function addon:ShowConfig()
	-- Open the profiles tab before, so the menu expands
	InterfaceOptionsFrame_OpenToCategory( self.optionsFrame["Display"] )
end

function addon:OnInitialize()
	-- I don't really do anything with onInit. I don't really truly "load"
	-- until I'm sure I'm in a guild (GRUs received). So just stub this.
end

function addon:CreateSVDB()
	self.db = LibStub( "AceDB-3.0" ):New( MODNAME .. "_DB", nil, "Default" )
	-- do we have any data yet; for this toon? If not, init its table
	if not ( self.db.factionrealm[addon.guild] ) then
		self.db.factionrealm[addon.guild] = {
		-- Your saved variables stuff here
			options = {
				display = {
					debug = false,
					tipscale = 1,
					dbVersion = "1.1",
				},
			},
			players = {
			},
			toons = {
			},
		}
	else
		-- check if we need to upgrade the db
		addon.upgradeDB()
	end
	db = self.db.factionrealm[addon.guild]
end

function addon:updateSorting()
	table.sort( addon.displayList, function ( x, y ) 
--		addon.Debug( "x/y" .. x .."/".. y )
		local dbentry = addon.db.factionrealm[addon.guild]
		if ( addon.sortAscend == 1 ) then
			
			if ( x[addon.sortMethod] <= y[addon.sortMethod] ) then
				return true
			else
				return false
			end
		else
			if ( y[addon.sortMethod] <= x[addon.sortMethod] ) then
				return true
			else
				return false
			end
		end
	end )
end

local function scanGuildInfo()
	local dbtn
	-- Gather guild info, start with the ranks
	addon.numGuildRanks = GuildControlGetNumRanks()
	addon.GuildRanks = {}
	for i = 1, addon.numGuildRanks do
		-- for each rank, get all the permissions info
		addon.GuildRanks[ i ] = {}
		GuildControlSetRank( i )
		addon.GuildRanks[ i ].Name = GuildControlGetRankName( i )
		local guildchat_listen, guildchat_speak, officerchat_listen, officerchat_speak, promote, demote,
			invite_member, remove_member, set_motd, edit_public_note, view_officer_note, edit_officer_note,
			modify_guild_info, _, withdraw_repair, withdraw_gold, create_guild_event = GuildControlGetRankFlags();
		addon.GuildRanks[ i ][1] = guildchat_listen
		addon.GuildRanks[ i ][2] = guildchat_speak
		addon.GuildRanks[ i ][3] = officerchat_listen
		addon.GuildRanks[ i ][4] = officerchat_speak
		addon.GuildRanks[ i ][5] = promote
		addon.GuildRanks[ i ][6] = demote
		addon.GuildRanks[ i ][7] = invite_member
		addon.GuildRanks[ i ][8] = remove_member
		addon.GuildRanks[ i ][9] = set_motd
		addon.GuildRanks[ i ][10] = edit_public_note
		addon.GuildRanks[ i ][11] = view_officer_note
		addon.GuildRanks[ i ][12] = edit_officer_note
		addon.GuildRanks[ i ][13] = modify_guild_info
		addon.GuildRanks[ i ][14] = withdraw_repair
		addon.GuildRanks[ i ][15] = withdraw_gold
		addon.GuildRanks[ i ][16] = create_guild_event
	end
	-- Get Guildie info
	addon.numGuildMembers = GetNumGuildMembers(true)
	for i = 1, addon.numGuildMembers do
		local name, rank, rankIndex, level, class, zone, note, 
			officernote, online, status, classFileName = GetGuildRosterInfo(i)
		-- process each toon
		if ( not db.toons[name] ) then
			db.toons[name] = {}
		end
		dbtn = db.toons[name]
		dbtn.rank = rank
		dbtn.rankIndex = rankIndex
		dbtn.level = level
		dbtn.class = class
		dbtn.enClass = classFileName
		dbtn.zone = zone
		dbtn.note = note
		dbtn.onote = officernote
		dbtn.status = status
		-- stick 'em in the list that'll get sorted
		addon.displayList[i] = {}
		addon.displayList[i].name = name
		addon.displayList[i].rank = rank
		addon.displayList[i].level = leve
		addon.displayList[i].class = class
		addon.displayList[i].enClass = classFileName
		addon.displayList[i].zone = zone
		addon.displayList[i].race = ""
	end
	addon.Debug( "found " .. addon.numGuildMembers .. " members" )
	addon:updateSorting()
end

function addon:GUILD_ROSTER_UPDATE()
	-- It doesn't appear to be necessary to wait for anything but the first GRU
	-- anymore, but just for good measure, I'm going to wait for the second one.
	if ( addon.GRUs < 2 ) then
		addon.GRUs = addon.GRUs + 1
	else
		-- if we're not in a guild, we don't do anything
		if ( IsInGuild() ) then
			-- Is this the first time we got here with this load? If so, we need to 
			-- go about setting up all our crap
			local GuildName, GuildRankName, GuildRankNum = GetGuildInfo( "player" )
			addon.guild = GuildName
			if ( addon.dataReady == false ) then
				addon.DDebug( "dotguild = " .. addon.guild )
				-- Frame Creation stuff here
				addon.DDebug( "first time through" )
				addon:CreateFrames()
				addon:InitializeList()
				addon:CreateSVDB()
				addon:SetupOptions()
				scanGuildInfo()
				addon:UpdateFrames()
				addon.dataReady = true
			else
				addon.Debug( "add'l GRU" )
				scanGuildInfo()
			end
		end
	end
end
		
--[[		

]]--
function addon:OnEnable()
    -- Called when the addon is loaded
	-- Very minimal stuff here since I don't know if I should
	-- load until I know I'm in a guild.

	-- Event listeners
    addon:RegisterEvent("GUILD_ROSTER_UPDATE")
end

--[[
************************************************************************
CHANGELOG:

08/10/09 :
	moved core functions and definitions to new files
5/1/09 : 
	Initial version
************************************************************************
]]--